@@ -319,15 +319,19 @@ class Playlist(QtCore.QAbstractTableModel):
                 self.slot_play_pause, 'media-playback-start', 'Ctrl+P', 'Ctrl+Alt+P',
                 factory=kdeui.KToggleAction)
 
+        # The following actions have their global shortcut set to the empty
+        # string. This allows the user to configure a global shortcut for these
+        # actions, without us having to provide a default one (and polluting
+        # the global shortcut namespace).
         self.action_stop = util.create_action('action_stop', 'Stop',
-                self.slot_stop, 'media-playback-stop', 'Ctrl+O',# 'Ctrl+Alt+I,O',
+                self.slot_stop, 'media-playback-stop', 'Ctrl+O', '',
                 factory=StopAction)
 
         self.action_next = util.create_action('action_next', 'Next',
-                self.slot_next, 'media-skip-forward', 'Ctrl+N')#, 'Ctrl+Alt+I,N')
+                self.slot_next, 'media-skip-forward', 'Ctrl+N', '')
 
         self.action_previous = util.create_action('action_previous', 'Previous',
-                self.slot_previous, 'media-skip-backward', 'Ctrl+I')#, 'Ctrl+Alt+I,P')
+                self.slot_previous, 'media-skip-backward', 'Ctrl+I', '')
 
         # Note: the icon here is named minirok_foo-bar and not minirok-foo-bar,
         # because if it isn't found, minirok-* seems to select the minirok.png
@@ -365,13 +369,15 @@ class Playlist(QtCore.QAbstractTableModel):
         self.redo_qaction = self.undo_stack.createRedoAction(self)
 
         self.connect(self.undo_qaction, QtCore.SIGNAL('changed()'),
-                     self.slot_undo_qaction_changed)
+            lambda: self.adjust_kaction_from_qaction(self.undo_kaction,
+                                                     self.undo_qaction))
 
         self.connect(self.redo_qaction, QtCore.SIGNAL('changed()'),
-                     self.slot_redo_qaction_changed)
+            lambda: self.adjust_kaction_from_qaction(self.redo_kaction,
+                                                     self.redo_qaction))
 
-        self.slot_undo_qaction_changed()
-        self.slot_redo_qaction_changed()
+        self.adjust_kaction_from_qaction(self.undo_kaction, self.undo_qaction)
+        self.adjust_kaction_from_qaction(self.redo_kaction, self.redo_qaction)
 
     ##
 
@@ -500,12 +506,6 @@ class Playlist(QtCore.QAbstractTableModel):
         if rows:
             self.my_emit_dataChanged(min(rows), max(rows))
 
-    def slot_undo_qaction_changed(self):
-        self.undo_kaction.setEnabled(self.undo_qaction.isEnabled())
-
-    def slot_redo_qaction_changed(self):
-        self.redo_kaction.setEnabled(self.redo_qaction.isEnabled())
-
     ##
 
     """Actions."""
@@ -874,6 +874,12 @@ class Playlist(QtCore.QAbstractTableModel):
         else:
             return {}
 
+    ##
+
+    def adjust_kaction_from_qaction(self, kaction, qaction):
+        kaction.setToolTip(qaction.text())
+        kaction.setEnabled(qaction.isEnabled())
+
 ##
 
 class Proxy(proxy.Model):
@@ -1006,6 +1012,10 @@ class PlaylistView(QtGui.QTreeView):
         self.setSelectionBehavior(self.SelectRows)
         self.setSelectionMode(self.ExtendedSelection)
 
+        self.action_queue_tracks = util.create_action(
+                'action_enqueue_dequeue_selected', 'Enqueue/dequeue selection',
+                self.slot_enqueue_dequeue_selected, None, 'Ctrl+E')
+
     def setModel(self, playlist):
         QtGui.QTreeView.setModel(self, playlist)
         self.header().setup_from_config()
@@ -1089,6 +1099,9 @@ class PlaylistView(QtGui.QTreeView):
             else:
                 return QtGui.QTreeView.mousePressEvent(self, event)
 
+        elif keymod != Qt.NoModifier: # Eat them up
+            return QtGui.QTreeView.mousePressEvent(self, event)
+
         elif button & Qt.MidButton:
             if index.data(Playlist.RoleQueryIsPlaying).toBool():
                 minirok.Globals.action_collection.action('action_pause').trigger()
@@ -1131,6 +1144,11 @@ class PlaylistView(QtGui.QTreeView):
         else:
             return QtGui.QTreeView.mousePressEvent(self, event)
 
+    ##
+
+    def slot_enqueue_dequeue_selected(self):
+        self.model().toggle_enqueued_many(sorted(self.uniqSelectedIndexes()))
+
 ##
 
 class PlaylistItem(object):
@@ -1139,8 +1157,6 @@ class PlaylistItem(object):
 
     ALLOWED_TAGS = [ 'Track', 'Artist', 'Album', 'Title', 'Length' ]
 
-    TRACK_COLUMN_INDEX = 0 # used by the model
-
     def __init__(self, path, tags=None):
         self.path = path
 
@@ -1230,7 +1246,8 @@ class PlaylistTrackDelegate(QtGui.QItemDelegate):
                 s_width = 8
                 s_height = 8
             else:
-                s_width = s_height = 0
+                s_width = 1 # Seems to prevent an artifact
+                s_height = 0
 
             if queue_pos:
                 queue_pos = str(queue_pos)
@@ -1271,8 +1288,10 @@ class Columns(QtGui.QHeaderView):
     # not.
     CONFIG_SECTION = 'Playlist'
     CONFIG_OPTION = 'Columns'
-    CONFIG_OPTION_DEFAULT = \
-            'Track:60:1,Artist:200:1,Album:200:0,Title:275:1,Length:60:1'
+    DEFAULT_COLUMNS = [
+        ('Track', 60, 1), ('Artist', 200, 1),
+        ('Album', 200, 0), ('Title', 275, 1), ('Length', 60, 1)
+    ]
 
     def __init__(self, parent):
         QtGui.QHeaderView.__init__(self, Qt.Horizontal, parent)
@@ -1302,54 +1321,52 @@ class Columns(QtGui.QHeaderView):
         a model/view associated with the object.
         """
         config = kdecore.KGlobal.config().group(self.CONFIG_SECTION)
+        columns = []
+        model_columns = self.sorted_column_names()
+        unseen_columns = set(model_columns)
 
-        if config.hasKey(self.CONFIG_OPTION):
+        if not config.hasKey(self.CONFIG_OPTION):
+            columns.extend(self.DEFAULT_COLUMNS)
+        else:
+            warn = minirok.logger.warn
             entries = map(str, config.readEntry(
                 self.CONFIG_OPTION,
                 QtCore.QVariant(QtCore.QStringList())).toStringList())
-        else:
-            entries = self.CONFIG_OPTION_DEFAULT.split(',')
 
-        columns = []
-        warn = minirok.logger.warn
-        model_columns = self.sorted_column_names()
-        known_columns = set(model_columns)
+            for entry in entries:
+                try:
+                    name, width, visible = entry.split(':', 2)
+                except ValueError:
+                    warn('skipping invalid entry in column config: %r', entry)
+                    continue
 
-        for entry in entries:
-            try:
-                name, width, visible = entry.split(':', 2)
-            except ValueError:
-                warn('skipping invalid entry in column config: %r', entry)
-                continue
+                try:
+                    width = int(width)
+                except ValueError:
+                    warn('invalid column width for %s: %r', name, width)
+                    continue
 
-            try:
-                width = int(width)
-            except ValueError:
-                warn('invalid column width for %s: %r', name, width)
-                continue
+                # TODO Maybe this one ought to be more flexible
+                try:
+                    visible = bool(int(visible))
+                except ValueError:
+                    warn('invalid visibility value for %s: %r', name, visible)
+                    continue
 
-            # TODO Maybe this one ought to be more flexible
-            try:
-                visible = bool(int(visible))
-            except ValueError:
-                warn('invalid visibility value for %s: %r', name, visible)
-                continue
+                try:
+                    unseen_columns.remove(name)
+                except KeyError:
+                    warn('skipping unknown or duplicated column: %r', name)
+                    continue
 
-            try:
-                known_columns.remove(name)
-            except KeyError:
-                warn('skipping unknown or duplicated column: %r', name)
-                continue
+                columns.append((name, width, visible))
 
-            columns.append((name, width, visible))
-
-        if len(known_columns) > 0:
-            defaults = dict((x.split(':')[0], map(int, x.split(':')[1:]))
-                                for x in self.CONFIG_OPTION_DEFAULT.split(','))
-            for c in known_columns:
-                warn('column %s missing in config, adding with default values', c)
-                width, visible = defaults[c]
-                columns.append((c, width, visible))
+            if unseen_columns:
+                missing = [ d for d in self.DEFAULT_COLUMNS
+                                if d[0] in unseen_columns ]
+                warn('these columns could not be found in config, creating from'
+                     ' defaults: %s', ', '.join(d[0] for d in missing))
+                columns.extend(missing)
 
         ##
 
