@@ -4,7 +4,6 @@
 # Copyright (c) 2007-2008 Adeodato Simó (dato@net.com.org.es)
 # Licensed under the terms of the MIT license.
 
-import qt
 import mutagen
 import mutagen.id3
 import mutagen.mp3
@@ -15,26 +14,11 @@ from minirok import util
 
 ##
 
-class TagReader(qt.QObject):
-    """Reads tags from files in a pending queue."""
+class TagReader(util.ThreadedWorker):
+    """Worker to read tags from files."""
 
     def __init__(self):
-        qt.QObject.__init__(self)
-
-        self.timer = qt.QTimer(self, 'tag reader timer')
-        self.connect(self.timer, qt.SIGNAL('timeout()'), self.update_done)
-
-        self.worker = util.ThreadedWorker(
-                lambda item: TagReader.tags(item.path), self.timer)
-        self.worker.start()
-
-    ##
-
-    def update_done(self):
-        for item, tags in self.worker.pop_done():
-            if tags:
-                item.update_tags(tags)
-                item.update_display()
+        util.ThreadedWorker.__init__(self, lambda item: self.tags(item.path))
 
     ##
 
